// 002 u_c_shape_list
// 28 feb 2009

// -- (C) John COLIBRI 2009
// -- documentation: http://www.jcolibri.com

(*$r+*)

unit u_c_shape_list;
  interface
    uses graphics;

    TYPE c_base_shape =
             Class
               m_x,m_y: integer;
               m_c_canvas_ref: tCanvas;

               Constructor create_base_shape(p_x, p_y: Integer;
                   p_c_canvas_ref: tCanvas);
               procedure draw_shape; Virtual; Abstract;
               procedure move_shape(p_delta_x, p_delta_y: Integer);
             end; // c_base_shape
         c_shape_list=
             Class
               m_shape_array: array of c_base_shape;
               m_shape_count: integer;

               Constructor create_shape_list;
               procedure draw_shape_list;
               procedure add_shape(p_c_base_shape: c_base_shape);
               procedure move_shape(p_delta_x, p_delta_y: Integer);
               Destructor Destroy; Override;
             end; // c_shape_list

  implementation

    // -- c_base_shape

    Constructor c_base_shape.create_base_shape(p_x, p_y: Integer;
        p_c_canvas_ref: tCanvas);
      begin
        m_x:= p_x;
        m_y:= p_y;

        m_c_canvas_ref:= p_c_canvas_ref;
      end; // create_base_shape

    procedure c_base_shape.move_shape(p_delta_x, p_delta_y: Integer);
      var l_save_pen_color: Integer;
      begin
        // -- "erase" by drawing with the background color
        l_save_pen_color:= m_c_canvas_ref.Pen.Color;
        // -- here clYellow, do show what's going on
        // --   but should use clBtnFace for the Form background
        m_c_canvas_ref.Pen.Color:= clYellow;
        draw_shape;

        Inc(m_x, p_delta_x);
        Inc(m_y, p_delta_y);

        m_c_canvas_ref.Pen.Color:= l_save_pen_color;

        draw_shape;
      end; // move_shape

    // -- c_shape_list

    Constructor c_shape_list.create_shape_list;
      begin
        SetLength(m_shape_array, 16);
      end; // create_shape_list

    procedure c_shape_list.add_shape(p_c_base_shape: c_base_shape);
      var l_array_length: Integer;
      begin
        // -- buddy / buddy
        l_array_length:= Length(m_shape_array);
        if m_shape_count= l_array_length
          then SetLength(m_shape_array, 2* l_array_length);

        m_shape_array[m_shape_count]:= p_c_base_shape;
        Inc(m_shape_count);

        p_c_base_shape.draw_shape;
      end; // add_shape

    procedure c_shape_list.draw_shape_list;
      var l_shape_index: Integer;
      begin
        For l_shape_index:= 0 to m_shape_count-1 do
          m_shape_array[l_shape_index].draw_shape;
      end; // draw_shape_list

    procedure c_shape_list.move_shape(p_delta_x, p_delta_y: Integer);
      var l_shape_index: Integer;
      begin
        For l_shape_index:= 0 to m_shape_count- 1 do
          m_shape_array[l_shape_index].move_shape(p_delta_x, p_delta_y);
      end; // move_shape

    Destructor c_shape_list.Destroy;
      var l_shape_index: Integer;
      begin
        For l_shape_index:= 0 to m_shape_count- 1 do
          m_shape_array[l_shape_index].Free;
        Inherited;
      end; // Destroy

end.




